/*
 * MSP430-Hardware-UART
 * ====================
 *   Uwe Berger; 2013
 * 
 * 
 * 
*/

#include "msp430.h"
#include <string.h>

#include "uart.h"

#define UART_RXBUFLEN 16                             // Groesse RX-Puffer

static volatile uint8_t uart_rxbuf[UART_RXBUFLEN];   // RX-Puffer
static volatile uint8_t uart_rxsize = 0;             // momentane Groesse RX-Puffer 

#ifndef F_CPU
#error "no define F_CPU!"
#endif

// *************************** 
void uart_init(uint16_t baud) {

	P1SEL = BIT1 + BIT2 ;  // P1.1 = RXD, P1.2=TXD
	P1SEL2 = BIT1 + BIT2 ; // P1.1 = RXD, P1.2=TXD
	UCA0CTL1 |= UCSSEL_2;  // SMCLK
	UCA0BR0 = F_CPU/baud;  // 
	UCA0BR1 = 0;           // 
	UCA0MCTL = UCBRS0;     // Modulation UCBRSx = 1
	UCA0CTL1 &= ~UCSWRST;  // **Initialize USCI state machine**
	IE2 |= UCA0RXIE;       // Enable USCI_A0 RX interrupt	
} 
 
// *************************** 
void uart_putc(uint8_t c) {
	while (!(IFG2 & UCA0TXIFG)); // USCI_A0 TX bereit?
	UCA0TXBUF = c;               // raus damit...!
}
 
// *************************** 
void uart_puts(const char *s) {
     while(*s) uart_putc(*s++);
} 

// *************************** 
uint8_t uart_getc(void) {
	static uint8_t uart_rxstart = 0; 
	uint8_t c;
	while (uart_rxsize == 0);                            // warten bis was im RX-Puffer steht...
	c = uart_rxbuf[uart_rxstart++];                      // Zeichen zurueckgeben ...
	if (uart_rxstart == UART_RXBUFLEN) uart_rxstart = 0; // ... und Puffer-"Zeiger" 
	uart_rxsize--;                                       // verwalten
	return c;
}

// *************************** 
uint8_t uart_get_line(char* buffer, uint8_t buffer_length) {
    memset(buffer, 0, buffer_length);
    uint8_t read_length = 0;
    while(read_length < buffer_length - 1)
    {
        uint8_t c = uart_getc();
        if(c == 0x08 || c == 0x7f)
        {
            if(read_length < 1) continue;
            --read_length;
            buffer[read_length] = '\0';
            uart_putc(0x08);
            uart_putc(' ');
            uart_putc(0x08);
            continue;
        }
        uart_putc(c);
        if(c == 0x0d)
        {
            buffer[read_length] = '\0';
            break;
        }
        else
        {
            buffer[read_length] = c;
            ++read_length;
        }
    }
    return read_length;
} 
 
// ***************************  
#pragma vector=USCIAB0RX_VECTOR
__interrupt void USCI0RX_ISR(void) {
	static uint8_t uart_rxstop  = 0;    
	uint8_t c;
	c = UCA0RXBUF;                     // empfangenes Zeichen einlesen...
	if (uart_rxsize < UART_RXBUFLEN) { // ...und in RX-Puffer schreiben
		uart_rxbuf[uart_rxstop++] = c;  
		if (uart_rxstop >= UART_RXBUFLEN) uart_rxstop = 0;  
		uart_rxsize++;     
	}
}
